import { $ } from 'vscode/vscode/vs/base/browser/dom';
import { alert } from 'vscode/vscode/vs/base/browser/ui/aria/aria';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { MarkdownString } from 'vscode/vscode/vs/base/common/htmlContent';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { ThemeIcon } from 'vscode/vscode/vs/base/common/themables';
import { isResponseVM } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatViewModel';

class ChatProgressContentPart extends Disposable {
    constructor(progress, renderer, context, forceShowSpinner, forceShowMessage, icon) {
        super();
        const followingContent = context.content.slice(context.contentIndex + 1);
        this.showSpinner = forceShowSpinner ?? shouldShowSpinner(followingContent, context.element);
        this.isHidden = forceShowMessage !== true && ( followingContent.some(part => part.kind !== 'progressMessage'));
        if (this.isHidden) {
            this.domNode = $('');
            return;
        }
        if (this.showSpinner) {
            alert(progress.content.value);
        }
        const codicon = icon ? icon.id : this.showSpinner ? ThemeIcon.modify(Codicon.loading, 'spin').id : Codicon.check.id;
        const markdown = ( new MarkdownString(`$(${codicon}) ${progress.content.value}`, {
            supportThemeIcons: true
        }));
        const result = this._register(renderer.render(markdown));
        result.element.classList.add('progress-step');
        this.domNode = result.element;
    }
    hasSameContent(other, followingContent, element) {
        if (( followingContent.some(part => part.kind !== 'progressMessage')) && !this.isHidden) {
            return false;
        }
        const showSpinner = shouldShowSpinner(followingContent, element);
        return other.kind === 'progressMessage' && this.showSpinner === showSpinner;
    }
}
function shouldShowSpinner(followingContent, element) {
    return isResponseVM(element) && !element.isComplete && followingContent.length === 0;
}

export { ChatProgressContentPart };
