import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { $ as $$1 } from 'vscode/vscode/vs/base/browser/dom';
import { CancellationTokenSource } from 'vscode/vscode/vs/base/common/cancellation';
import { Emitter } from 'vscode/vscode/vs/base/common/event';
import { Disposable, toDisposable } from 'vscode/vscode/vs/base/common/lifecycle';
import 'vscode/vscode/vs/base/common/network';
import 'vscode/vscode/vs/base/common/resources';
import { assertType } from 'vscode/vscode/vs/base/common/types';
import 'vscode/vscode/vs/base/common/path';
import 'vscode/vscode/vs/base/common/platform';
import 'vscode/vscode/vs/editor/common/languages';
import 'vscode/vscode/vs/editor/common/model/textModel';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import 'vscode/vscode/vs/editor/common/services/modelService';
import { localize } from 'vscode/vscode/vs/nls';
import { MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ResourcePool } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/chatContentParts/chatCollections';
import { CodeCompareBlockPart } from '../codeBlockPart.js';
import { isResponseVM } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatViewModel';
import { ICodeCompareModelService } from './chatTextEditContentPart.service.js';

const $ = $$1;
let ChatTextEditContentPart = class ChatTextEditContentPart extends Disposable {
    constructor(chatTextEdit, context, rendererOptions, diffEditorPool, currentWidth, codeCompareModelService) {
        super();
        this.codeCompareModelService = codeCompareModelService;
        this._onDidChangeHeight = this._register(( (new Emitter())));
        this.onDidChangeHeight = this._onDidChangeHeight.event;
        const element = context.element;
        assertType(isResponseVM(element));
        if (rendererOptions.renderTextEditsAsSummary?.(chatTextEdit.uri)) {
            if (element.response.value.every(item => item.kind === 'textEditGroup')) {
                this.domNode = $('.interactive-edits-summary', undefined, !element.isComplete
                    ? ''
                    : element.isCanceled
                        ? ( localize(10844, "Making changes was aborted."))
                        : ( localize(10845, "Made changes.")));
            }
            else {
                this.domNode = $('div');
            }
        }
        else {
            const cts = ( (new CancellationTokenSource()));
            let isDisposed = false;
            this._register(toDisposable(() => {
                isDisposed = true;
                cts.dispose(true);
            }));
            this.comparePart = this._register(diffEditorPool.get());
            this._register(this.comparePart.object.onDidChangeContentHeight(() => {
                this._onDidChangeHeight.fire();
            }));
            const data = {
                element,
                edit: chatTextEdit,
                diffData: (async () => {
                    const ref = await this.codeCompareModelService.createModel(element, chatTextEdit);
                    if (isDisposed) {
                        ref.dispose();
                        return;
                    }
                    this._register(ref);
                    return {
                        modified: ref.object.modified.textEditorModel,
                        original: ref.object.original.textEditorModel,
                        originalSha1: ref.object.originalSha1
                    };
                })()
            };
            this.comparePart.object.render(data, currentWidth, cts.token);
            this.domNode = this.comparePart.object.element;
        }
    }
    layout(width) {
        this.comparePart?.object.layout(width);
    }
    hasSameContent(other) {
        return other.kind === 'textEditGroup';
    }
    addDisposable(disposable) {
        this._register(disposable);
    }
};
ChatTextEditContentPart = ( (__decorate([
    ( (__param(5, ICodeCompareModelService)))
], ChatTextEditContentPart)));
let DiffEditorPool = class DiffEditorPool extends Disposable {
    inUse() {
        return this._pool.inUse;
    }
    constructor(options, delegate, overflowWidgetsDomNode, instantiationService) {
        super();
        this._pool = this._register(( (new ResourcePool(() => {
            return instantiationService.createInstance(CodeCompareBlockPart, options, MenuId.ChatCompareBlock, delegate, overflowWidgetsDomNode);
        }))));
    }
    get() {
        const codeBlock = this._pool.get();
        let stale = false;
        return {
            object: codeBlock,
            isStale: () => stale,
            dispose: () => {
                codeBlock.reset();
                stale = true;
                this._pool.release(codeBlock);
            }
        };
    }
};
DiffEditorPool = ( (__decorate([
    ( (__param(3, IInstantiationService)))
], DiffEditorPool)));

export { ChatTextEditContentPart, DiffEditorPool };
