import { localize } from 'vscode/vscode/vs/nls';
import { MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { Extensions } from 'vscode/vscode/vs/platform/configuration/common/configurationRegistry';
import { RawContextKey } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { registerColor, transparent } from 'vscode/vscode/vs/platform/theme/common/colorUtils';
import { focusBorder } from 'vscode/vscode/vs/platform/theme/common/colors/baseColors';
import 'vscode/vscode/vs/platform/theme/common/colors/chartsColors';
import { editorWidgetForeground, editorWidgetBackground, editorWidgetBorder, widgetShadow, diffInserted, diffRemoved } from 'vscode/vscode/vs/platform/theme/common/colors/editorColors';
import { inputPlaceholderForeground, inputBackground } from 'vscode/vscode/vs/platform/theme/common/colors/inputColors';
import 'vscode/vscode/vs/platform/theme/common/colors/listColors';
import 'vscode/vscode/vs/platform/theme/common/colors/menuColors';
import 'vscode/vscode/vs/platform/theme/common/colors/minimapColors';
import 'vscode/vscode/vs/platform/theme/common/colors/miscColors';
import 'vscode/vscode/vs/platform/theme/common/colors/quickpickColors';
import 'vscode/vscode/vs/platform/theme/common/colors/searchColors';

( (Registry.as(Extensions.Configuration))).registerConfiguration({
    id: 'editor',
    properties: {
        ["inlineChat.mode" ]: {
            description: ( localize(
                7440,
                "Configure if changes crafted with inline chat are applied directly to the document or are previewed first."
            )),
            default: "live" ,
            type: 'string',
            enum: ["live" , "preview" ],
            markdownEnumDescriptions: [
                ( localize(
                    7441,
                    "Changes are applied directly to the document, can be highlighted via inline diffs, and accepted/discarded by hunks. Ending a session will keep the changes."
                )),
                ( localize(
                    7442,
                    "Changes are previewed only and need to be accepted via the apply button. Ending a session will discard the changes."
                )),
            ]
        },
        ["inlineChat.finishOnType" ]: {
            description: ( localize(
                7443,
                "Whether to finish an inline chat session when typing outside of changed regions."
            )),
            default: false,
            type: 'boolean'
        },
        ["inlineChat.holdToSpeech" ]: {
            description: ( localize(
                7444,
                "Whether holding the inline chat keybinding will automatically enable speech recognition."
            )),
            default: true,
            type: 'boolean'
        },
        ["inlineChat.accessibleDiffView" ]: {
            description: ( localize(
                7445,
                "Whether the inline chat also renders an accessible diff viewer for its changes."
            )),
            default: 'auto',
            type: 'string',
            enum: ['auto', 'on', 'off'],
            markdownEnumDescriptions: [
                ( localize(
                    7446,
                    "The accessible diff viewer is based on screen reader mode being enabled."
                )),
                ( localize(7447, "The accessible diff viewer is always enabled.")),
                ( localize(7448, "The accessible diff viewer is never enabled.")),
            ],
        },
    }
});
const INLINE_CHAT_ID = 'interactiveEditor';
const CTX_INLINE_CHAT_POSSIBLE = ( (new RawContextKey('inlineChatPossible', false, ( localize(
    7449,
    "Whether a provider for inline chat exists and whether an editor for inline chat is open"
)))));
const CTX_INLINE_CHAT_HAS_AGENT = ( (new RawContextKey('inlineChatHasProvider', false, ( localize(7450, "Whether a provider for interactive editors exists")))));
const CTX_INLINE_CHAT_VISIBLE = ( (new RawContextKey('inlineChatVisible', false, ( localize(7451, "Whether the interactive editor input is visible")))));
const CTX_INLINE_CHAT_FOCUSED = ( (new RawContextKey('inlineChatFocused', false, ( localize(7452, "Whether the interactive editor input is focused")))));
const CTX_INLINE_CHAT_EDITING = ( (new RawContextKey('inlineChatEditing', true, ( localize(
    7453,
    "Whether the user is currently editing or generating code in the inline chat"
)))));
const CTX_INLINE_CHAT_RESPONSE_FOCUSED = ( (new RawContextKey('inlineChatResponseFocused', false, ( localize(7454, "Whether the interactive widget's response is focused")))));
const CTX_INLINE_CHAT_EMPTY = ( (new RawContextKey('inlineChatEmpty', false, ( localize(7455, "Whether the interactive editor input is empty")))));
const CTX_INLINE_CHAT_INNER_CURSOR_FIRST = ( (new RawContextKey('inlineChatInnerCursorFirst', false, ( localize(
    7456,
    "Whether the cursor of the iteractive editor input is on the first line"
)))));
const CTX_INLINE_CHAT_INNER_CURSOR_LAST = ( (new RawContextKey('inlineChatInnerCursorLast', false, ( localize(
    7457,
    "Whether the cursor of the iteractive editor input is on the last line"
)))));
const CTX_INLINE_CHAT_OUTER_CURSOR_POSITION = ( (new RawContextKey('inlineChatOuterCursorPosition', '', ( localize(
    7460,
    "Whether the cursor of the outer editor is above or below the interactive editor input"
)))));
const CTX_INLINE_CHAT_HAS_STASHED_SESSION = ( (new RawContextKey('inlineChatHasStashedSession', false, ( localize(7461, "Whether interactive editor has kept a session for quick restore")))));
const CTX_INLINE_CHAT_USER_DID_EDIT = ( (new RawContextKey('inlineChatUserDidEdit', undefined, ( localize(7462, "Whether the user did changes ontop of the inline chat")))));
const CTX_INLINE_CHAT_DOCUMENT_CHANGED = ( (new RawContextKey('inlineChatDocumentChanged', false, ( localize(7463, "Whether the document has changed concurrently")))));
const CTX_INLINE_CHAT_CHANGE_HAS_DIFF = ( (new RawContextKey('inlineChatChangeHasDiff', false, ( localize(7464, "Whether the current change supports showing a diff")))));
const CTX_INLINE_CHAT_CHANGE_SHOWS_DIFF = ( (new RawContextKey('inlineChatChangeShowsDiff', false, ( localize(7465, "Whether the current change showing a diff")))));
const CTX_INLINE_CHAT_EDIT_MODE = ( (new RawContextKey('config.inlineChat.mode', "live" )));
const CTX_INLINE_CHAT_REQUEST_IN_PROGRESS = ( (new RawContextKey('inlineChatRequestInProgress', false, ( localize(7466, "Whether an inline chat request is currently in progress")))));
const CTX_INLINE_CHAT_RESPONSE_TYPE = ( (new RawContextKey(
    'inlineChatResponseType',
    "none" ,
    ( localize(
        7467,
        "What type was the responses have been receieved, nothing yet, just messages, or messaged and local edits"
    ))
)));
const ACTION_ACCEPT_CHANGES = 'inlineChat.acceptChanges';
const ACTION_DISCARD_CHANGES = 'inlineChat.discardHunkChange';
const ACTION_REGENERATE_RESPONSE = 'inlineChat.regenerate';
const ACTION_VIEW_IN_CHAT = 'inlineChat.viewInChat';
const ACTION_TOGGLE_DIFF = 'inlineChat.toggleDiff';
const ACTION_REPORT_ISSUE = 'inlineChat.reportIssue';
const MENU_INLINE_CHAT_WIDGET_STATUS = MenuId.for('inlineChatWidget.status');
const MENU_INLINE_CHAT_WIDGET_SECONDARY = MenuId.for('inlineChatWidget.secondary');
const MENU_INLINE_CHAT_ZONE = MenuId.for('inlineChatWidget.changesZone');
const inlineChatForeground = registerColor('inlineChat.foreground', editorWidgetForeground, ( localize(7468, "Foreground color of the interactive editor widget")));
const inlineChatBackground = registerColor('inlineChat.background', editorWidgetBackground, ( localize(7469, "Background color of the interactive editor widget")));
registerColor('inlineChat.border', editorWidgetBorder, ( localize(7470, "Border color of the interactive editor widget")));
registerColor('inlineChat.shadow', widgetShadow, ( localize(7471, "Shadow color of the interactive editor widget")));
registerColor('inlineChatInput.border', editorWidgetBorder, ( localize(7472, "Border color of the interactive editor input")));
registerColor('inlineChatInput.focusBorder', focusBorder, ( localize(7473, "Border color of the interactive editor input when focused")));
registerColor('inlineChatInput.placeholderForeground', inputPlaceholderForeground, ( localize(7474, "Foreground color of the interactive editor input placeholder")));
registerColor('inlineChatInput.background', inputBackground, ( localize(7475, "Background color of the interactive editor input")));
registerColor('inlineChatDiff.inserted', ( (transparent(diffInserted, .5))), ( localize(7476, "Background color of inserted text in the interactive editor input")));
const overviewRulerInlineChatDiffInserted = registerColor('editorOverviewRuler.inlineChatInserted', { dark: ( (transparent(diffInserted, 0.6))), light: ( (transparent(diffInserted, 0.8))), hcDark: ( (transparent(diffInserted, 0.6))), hcLight: ( (transparent(diffInserted, 0.8))) }, ( localize(7477, 'Overview ruler marker color for inline chat inserted content.')));
const minimapInlineChatDiffInserted = registerColor('editorOverviewRuler.inlineChatInserted', { dark: ( (transparent(diffInserted, 0.6))), light: ( (transparent(diffInserted, 0.8))), hcDark: ( (transparent(diffInserted, 0.6))), hcLight: ( (transparent(diffInserted, 0.8))) }, ( localize(7477, 'Overview ruler marker color for inline chat inserted content.')));
registerColor('inlineChatDiff.removed', ( (transparent(diffRemoved, .5))), ( localize(7478, "Background color of removed text in the interactive editor input")));
registerColor('editorOverviewRuler.inlineChatRemoved', { dark: ( (transparent(diffRemoved, 0.6))), light: ( (transparent(diffRemoved, 0.8))), hcDark: ( (transparent(diffRemoved, 0.6))), hcLight: ( (transparent(diffRemoved, 0.8))) }, ( localize(7479, 'Overview ruler marker color for inline chat removed content.')));

export { ACTION_ACCEPT_CHANGES, ACTION_DISCARD_CHANGES, ACTION_REGENERATE_RESPONSE, ACTION_REPORT_ISSUE, ACTION_TOGGLE_DIFF, ACTION_VIEW_IN_CHAT, CTX_INLINE_CHAT_CHANGE_HAS_DIFF, CTX_INLINE_CHAT_CHANGE_SHOWS_DIFF, CTX_INLINE_CHAT_DOCUMENT_CHANGED, CTX_INLINE_CHAT_EDITING, CTX_INLINE_CHAT_EDIT_MODE, CTX_INLINE_CHAT_EMPTY, CTX_INLINE_CHAT_FOCUSED, CTX_INLINE_CHAT_HAS_AGENT, CTX_INLINE_CHAT_HAS_STASHED_SESSION, CTX_INLINE_CHAT_INNER_CURSOR_FIRST, CTX_INLINE_CHAT_INNER_CURSOR_LAST, CTX_INLINE_CHAT_OUTER_CURSOR_POSITION, CTX_INLINE_CHAT_POSSIBLE, CTX_INLINE_CHAT_REQUEST_IN_PROGRESS, CTX_INLINE_CHAT_RESPONSE_FOCUSED, CTX_INLINE_CHAT_RESPONSE_TYPE, CTX_INLINE_CHAT_USER_DID_EDIT, CTX_INLINE_CHAT_VISIBLE, INLINE_CHAT_ID, MENU_INLINE_CHAT_WIDGET_SECONDARY, MENU_INLINE_CHAT_WIDGET_STATUS, MENU_INLINE_CHAT_ZONE, inlineChatBackground, inlineChatForeground, minimapInlineChatDiffInserted, overviewRulerInlineChatDiffInserted };
